==> trimica-1.4.3%2B1.21.10-fabric-sources.jar: com/bawnorton/trimica/client/mixin/model/BlockModelWrapperMixin.java <==
package com.bawnorton.trimica.client.mixin.model;

import com.bawnorton.trimica.Trimica;
import com.bawnorton.trimica.client.TrimicaClient;
import com.bawnorton.trimica.client.extend.ItemStackRenderState$LayerRenderStateExtension;
import com.bawnorton.trimica.client.extend.ItemStackRenderState.LayerRenderStateExtension;
import com.bawnorton.trimica.client.mixin.accessor.BlockModelWrapperAccessor;
import com.bawnorton.trimica.client.model.TrimItemModelFactory;
import com.bawnorton.trimica.client.model.TrimmedItemModelWrapper;
import com.bawnorton.trimica.client.palette.TrimPalette;
import com.bawnorton.trimica.item.component.AdditionalTrims;
import com.llamalad7.mixinextras.injector.wrapmethod.WrapMethod;
import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import dev.kikugie.fletching_table.annotation.MixinEnvironment;
import org.joml.Vector3f;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Mutable;
import org.spongepowered.asm.mixin.Shadow;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Supplier;
import net.minecraft.class_10209;
import net.minecraft.class_10401;
import net.minecraft.class_10430;
import net.minecraft.class_10439;
import net.minecraft.class_10442;
import net.minecraft.class_10444;
import net.minecraft.class_10809;
import net.minecraft.class_11566;
import net.minecraft.class_1799;
import net.minecraft.class_1921;
import net.minecraft.class_2960;
import net.minecraft.class_3695;
import net.minecraft.class_638;
import net.minecraft.class_777;
import net.minecraft.class_8053;
import net.minecraft.class_811;

@MixinEnvironment("client")
@Mixin(class_10430.class)
abstract class BlockModelWrapperMixin {
	@Shadow
	@Final
	@Mutable
	private List<class_10401> tints;

	@Shadow
	@Final
	@Mutable
	private List<class_777> quads;

	@Shadow
	@Final
	@Mutable
	private Supplier<Vector3f[]> extents;

	@Shadow
	@Final
	@Mutable
	private class_10809 properties;

	@WrapMethod(
			method = "update"
	)
	//? if >=1.21.10 {
	private void createNewLayerRenderStateForTrimOverlay(class_10444 renderState, class_1799 stack, class_10442 itemModelResolver, class_811 displayContext, class_638 level, class_11566 owner, int seed, Operation<Void> original) {
		Runnable callOriginal = () -> original.call(renderState, stack, itemModelResolver, displayContext, level, owner, seed);
	//?} else {
	/*private void createNewLayerRenderStateForTrimOverlay(ItemStackRenderState renderState, ItemStack stack, ItemModelResolver itemModelResolver, ItemDisplayContext displayContext, ClientLevel level, LivingEntity livingEntity, int i, Operation<Void> original) {
		Runnable callOriginal = () -> original.call(renderState, stack, itemModelResolver, displayContext, level, livingEntity, i);
	*///?}

		List<class_8053> trims = AdditionalTrims.getAllTrims(stack);
		if (trims.isEmpty()) {
			callOriginal.run();
			return;
		}

		class_3695 profiler = class_10209.method_64146();
		profiler.method_15396("trimica:overlay");
		boolean isEmissive = false;
		boolean isAnimated = false;
		List<TrimmedItemModelWrapper> trimModelWrappers = new ArrayList<>();
		for (class_8053 trim : trims) {
			profiler.method_15396("trimica:item_runtime_atlas");
			TrimItemModelFactory itemModelFactory = TrimicaClient.getItemModelFactory();
			TrimmedItemModelWrapper newModel = itemModelFactory.getOrCreateModel((class_10439) this, level, stack, trim);
			trimModelWrappers.add(newModel);
			profiler.method_15407();
			TrimPalette palette = newModel.palette();
			if (palette != null) {
				if (palette.isEmissive()) {
					isEmissive = true;
				}
				if (palette.isAnimated()) {
					isAnimated = true;
				}
			}
		}

		if (isAnimated) {
			renderState.method_70947();
		}
		class_10444.class_10445 foilType = class_10444.class_10445.field_55343;
		if (stack.method_7958()) {
			renderState.method_70946(foilType);
		}

		for (TrimmedItemModelWrapper newModel : trimModelWrappers) {
			List<class_10401> originalTints = new ArrayList<>(tints);
			List<class_777> originalQuads = new ArrayList<>(quads);
			Vector3f[] originalExtents = extents.get();
			Supplier<Vector3f[]> originalExtentsSupplier = () -> originalExtents;
			class_10809 originalProperties = new class_10809(
					properties.comp_3766(),
					properties.comp_3767(),
					properties.comp_3768()
			);
			if (newModel.model() instanceof BlockModelWrapperAccessor blockModelWrapper) {
				tints = blockModelWrapper.trimica$tints();
				quads = blockModelWrapper.trimica$quads();
				extents = blockModelWrapper.trimica$extents();
				properties = blockModelWrapper.trimica$properties();
			}

			int lastBaseTextureIndex = -1;
			class_2960 overlayAtlas = null;
			for (int j = 0; j < quads.size(); j++) {
				class_777 quad = quads.get(j);
				if (quad.comp_3724().method_45852().method_12836().equals(Trimica.MOD_ID)) {
					lastBaseTextureIndex = j;
					overlayAtlas = quad.comp_3724().method_45852();
					break;
				}
			}
			if (overlayAtlas == null) {
				tints = originalTints;
				quads = originalQuads;
				extents = originalExtentsSupplier;
				properties = originalProperties;
				callOriginal.run();
				return;
			}

			List<class_777> overlayQuads = new ArrayList<>(quads.subList(lastBaseTextureIndex, quads.size()));
			quads = new ArrayList<>(quads.subList(0, lastBaseTextureIndex));
			callOriginal.run();

			class_10444.class_10446 overlayRenderState = renderState.method_65601();
			LayerRenderStateExtension extender = (LayerRenderStateExtension) overlayRenderState;
			extender.trimica$markAsTrimOverlay();
			class_2960 modelLocation = newModel.location();
			renderState.method_70946(modelLocation);
			extender.trimica$setEmissive(isEmissive);
			if (stack.method_7958()) {
				overlayRenderState.method_65615(foilType);
			}

			overlayRenderState.method_67992(class_1921.method_29379(overlayAtlas));
			properties.method_68000(overlayRenderState, displayContext);
			overlayRenderState.method_67997().addAll(overlayQuads);

			quads = originalQuads;
			profiler.method_15407();
		}
	}
}

==> trimica-1.4.4%2B1.21.8-fabric-sources.jar: com/bawnorton/trimica/client/mixin/model/BlockModelWrapperMixin.java <==
package com.bawnorton.trimica.client.mixin.model;

import com.bawnorton.trimica.Trimica;
import com.bawnorton.trimica.client.TrimicaClient;
import com.bawnorton.trimica.client.extend.ItemStackRenderState$LayerRenderStateExtension;
import com.bawnorton.trimica.client.extend.ItemStackRenderState.LayerRenderStateExtension;
import com.bawnorton.trimica.client.mixin.accessor.BlockModelWrapperAccessor;
import com.bawnorton.trimica.client.model.TrimItemModelFactory;
import com.bawnorton.trimica.client.model.TrimmedItemModelWrapper;
import com.bawnorton.trimica.client.palette.TrimPalette;
import com.bawnorton.trimica.item.component.AdditionalTrims;
import com.llamalad7.mixinextras.injector.wrapmethod.WrapMethod;
import com.llamalad7.mixinextras.injector.wrapoperation.Operation;
import dev.kikugie.fletching_table.annotation.MixinEnvironment;
import org.joml.Vector3f;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Mutable;
import org.spongepowered.asm.mixin.Shadow;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Supplier;
import net.minecraft.class_10209;
import net.minecraft.class_10401;
import net.minecraft.class_10430;
import net.minecraft.class_10439;
import net.minecraft.class_10442;
import net.minecraft.class_10444;
import net.minecraft.class_10809;
import net.minecraft.class_1309;
import net.minecraft.class_1799;
import net.minecraft.class_1921;
import net.minecraft.class_2960;
import net.minecraft.class_3695;
import net.minecraft.class_638;
import net.minecraft.class_777;
import net.minecraft.class_8053;
import net.minecraft.class_811;

//? if >=1.21.10
/*import net.minecraft.world.entity.ItemOwner;*/

@MixinEnvironment("client")
@Mixin(class_10430.class)
abstract class BlockModelWrapperMixin {
	@Shadow
	@Final
	@Mutable
	private List<class_10401> tints;

	@Shadow
	@Final
	@Mutable
	private List<class_777> quads;

	@Shadow
	@Final
	@Mutable
	private Supplier<Vector3f[]> extents;

	@Shadow
	@Final
	@Mutable
	private class_10809 properties;

	@WrapMethod(
			method = "update"
	)
	//? if >=1.21.10 {
	/*private void createNewLayerRenderStateForTrimOverlay(ItemStackRenderState renderState, ItemStack stack, ItemModelResolver itemModelResolver, ItemDisplayContext displayContext, ClientLevel level, ItemOwner owner, int seed, Operation<Void> original) {
		Runnable callOriginal = () -> original.call(renderState, stack, itemModelResolver, displayContext, level, owner, seed);
	*///?} else {
	private void createNewLayerRenderStateForTrimOverlay(class_10444 renderState, class_1799 stack, class_10442 itemModelResolver, class_811 displayContext, class_638 level, class_1309 livingEntity, int i, Operation<Void> original) {
		Runnable callOriginal = () -> original.call(renderState, stack, itemModelResolver, displayContext, level, livingEntity, i);
	//?}

		List<class_8053> trims = AdditionalTrims.getAllTrims(stack);
		if (trims.isEmpty()) {
			callOriginal.run();
			return;
		}

		class_3695 profiler = class_10209.method_64146();
		profiler.method_15396("trimica:overlay");
		boolean isEmissive = false;
		boolean isAnimated = false;
		List<TrimmedItemModelWrapper> trimModelWrappers = new ArrayList<>();
		for (class_8053 trim : trims) {
			profiler.method_15396("trimica:item_runtime_atlas");
			TrimItemModelFactory itemModelFactory = TrimicaClient.getItemModelFactory();
			TrimmedItemModelWrapper newModel = itemModelFactory.getOrCreateModel((class_10439) this, level, stack, trim);
			trimModelWrappers.add(newModel);
			profiler.method_15407();
			TrimPalette palette = newModel.palette();
			if (palette != null) {
				if (palette.isEmissive()) {
					isEmissive = true;
				}
				if (palette.isAnimated()) {
					isAnimated = true;
				}
			}
		}

		if (isAnimated) {
			renderState.method_70947();
		}
		class_10444.class_10445 foilType = class_10444.class_10445.field_55343;
		if (stack.method_7958()) {
			renderState.method_70946(foilType);
		}

		for (TrimmedItemModelWrapper newModel : trimModelWrappers) {
			List<class_10401> originalTints = new ArrayList<>(tints);
			List<class_777> originalQuads = new ArrayList<>(quads);
			Vector3f[] originalExtents = extents.get();
			Supplier<Vector3f[]> originalExtentsSupplier = () -> originalExtents;
			class_10809 originalProperties = new class_10809(
					properties.comp_3766(),
					properties.comp_3767(),
					properties.comp_3768()
			);
			if (newModel.model() instanceof BlockModelWrapperAccessor blockModelWrapper) {
				tints = blockModelWrapper.trimica$tints();
				quads = blockModelWrapper.trimica$quads();
				extents = blockModelWrapper.trimica$extents();
				properties = blockModelWrapper.trimica$properties();
			}

			int lastBaseTextureIndex = -1;
			class_2960 overlayAtlas = null;
			for (int j = 0; j < quads.size(); j++) {
				class_777 quad = quads.get(j);
				if (quad.comp_3724().method_45852().method_12836().equals(Trimica.MOD_ID)) {
					lastBaseTextureIndex = j;
					overlayAtlas = quad.comp_3724().method_45852();
					break;
				}
			}
			if (overlayAtlas == null) {
				tints = originalTints;
				quads = originalQuads;
				extents = originalExtentsSupplier;
				properties = originalProperties;
				callOriginal.run();
				return;
			}

			List<class_777> overlayQuads = new ArrayList<>(quads.subList(lastBaseTextureIndex, quads.size()));
			quads = new ArrayList<>(quads.subList(0, lastBaseTextureIndex));
			callOriginal.run();

			class_10444.class_10446 overlayRenderState = renderState.method_65601();
			LayerRenderStateExtension extender = (LayerRenderStateExtension) overlayRenderState;
			extender.trimica$markAsTrimOverlay();
			class_2960 modelLocation = newModel.location();
			renderState.method_70946(modelLocation);
			extender.trimica$setEmissive(isEmissive);
			if (stack.method_7958()) {
				overlayRenderState.method_65615(foilType);
			}

			overlayRenderState.method_67992(class_1921.method_29379(overlayAtlas));
			properties.method_68000(overlayRenderState, displayContext);
			overlayRenderState.method_67997().addAll(overlayQuads);

			quads = originalQuads;
			profiler.method_15407();
		}
	}
}
